/*
    Copyright (c) 2004-2007 Markus Kettunen

    This file is part of Blobtrix.

    Blobtrix is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Blobtrix is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Blobtrix; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef WIN32
#include <sys/types.h>
#include <sys/stat.h>
#endif
#include "path.h"

path::path() {
	datadir = INSTALL_FILEDIR"/data";
	fontdir = INSTALL_FILEDIR"/fonts";

#ifdef WIN32
	homedir = "./";
#else
	char *homeenv = getenv("HOME");
	if (!homeenv) {
		fprintf (stderr, "Error: $HOME not set.\n");
		exit(1);
	}
	homedir = homeenv;
	homedir += "/.blobtrix";
	mkdir(homedir.c_str(), S_IRUSR | S_IWUSR | S_IXUSR);
#endif
}

path::~path() {
}

char *path::data(char *s) {
	if ( strlen(s) >= 1024 ) {
		fprintf (stderr, "Error: In path::data(char *s), strlen(s) must be < 1024.\n");
		exit(1);
	}

	sprintf (strtemp, "%s/%s", datadir.c_str(), s);
	return strtemp;
}

char *path::font(char *s) {
	if ( strlen(s) >= 1024 ) {
		fprintf (stderr, "Error: In path::font(char *s), strlen(s) must be < 1024.\n");
		exit(1);
	}

	sprintf (strtemp, "%s/%s", fontdir.c_str(), s);
	return strtemp;
}

char *path::home(char *s) {
	if ( strlen(s) >= 1024 ) {
		fprintf (stderr, "Error: In path::home(char *s), strlen(s) must be < 1024.\n");
		exit(1);
	}

	sprintf (strtemp, "%s/%s", homedir.c_str(), s);
	return strtemp;	
}
